/*
 *  puzzlespanel.c
 * 
 *  Author: George Vlahavas (vlahavas~AT~gmail~DOT~com, gapan@zenwalk forums)
 * 
 *  This is just a hacked version of Zenpanel to create a launcher for all
 *  the puzzles in this collection. All credit should be attributed to them.
 * 
 *  Zenpanel authors : 
 * - Vincent Kergonna <vincent.k@orange.fr> 
 * - Jean-Philippe Guillemin <jp.guillemin@free.fr>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include "puzzlespanel.h"
#include "ppnl_parser.h"

/* usage */
int usage(void) {
	printf("\npuzzlespanel v%s : A launcher for Simon Tatham's Portable Puzzle Collection\n", VERSION);
	printf("Usage : puzzlespanel [-w width] [-h height] -d filename\n");
	printf("[-W] : window Width\n");
	printf("[-H] : window Height\n");
	return(EXIT_SUCCESS);
}

static char default_desktop_dir[] = {_PUZZLEGAMESPATH};


gchar *desktop_dir = default_desktop_dir;
gint width = 530;
gint height = 540;


enum {
	ICON_COLUMN,
	TEXT_COLUMN,
	DATA_COLUMN
};


static void destroy (GtkWidget *widget, gpointer data){
	gtk_main_quit();
}

void child_exited (pid_t pid)
{
	pid = waitpid(-1, NULL, WNOHANG);
}

static void icon_activated (GtkIconView *view, GtkTreePath *tree, gpointer data){
	pid_t pid;
	GtkListStore *store;
	GtkTreeIter iter;
	gchar *cmd = NULL;
	char *argv[64];
	store = GTK_LIST_STORE (data);

	gtk_tree_model_get_iter (GTK_TREE_MODEL (store), &iter, tree);
	gtk_tree_model_get (GTK_TREE_MODEL (store), &iter,
                     DATA_COLUMN, &cmd, -1);
	
	ppnl_parse_cmd(cmd, argv); 
                     
	dbg("cmd=%s\n",cmd);
	pid = fork();
	if (pid == 0) {
		if (execvp(*argv, argv)) {
			exit(EXIT_FAILURE);
		}
	} else if (pid < 0) {
		dbg("Unable to fork");   
	} else if (pid > 0) {
		return;
	}
	return;
}


static void add_item_to_store (GtkListStore *store,
								gchar *icon_name,
								gchar *text,
								gchar *cmd){
					
	GtkTreeIter iter;
	GtkIconTheme *icon_theme;
	GdkPixbuf *pixbuf = NULL;
	
	if (!icon_name || !text || !cmd) {
		return;
	}
	
	icon_theme = gtk_icon_theme_get_default();	
	gtk_list_store_append (store, &iter);
	
	pixbuf = gtk_icon_theme_load_icon (icon_theme,
                                   icon_name, /* icon name */
                                   48, /* size */
                                   0,  /* flags */
                                   NULL);
	
	if (pixbuf) {
		gtk_list_store_set (store, &iter, ICON_COLUMN, pixbuf, 
			TEXT_COLUMN, text, DATA_COLUMN, cmd, -1);
		g_object_unref (pixbuf);
	} else {
		dbg("cannot create pixbuf from icon %s\n", icon_name);
	}
}

/*
 * This function reads .desktop files from a given directory and create the
 * icons in the panel.
 * 
 */
void
display_icons_in_panel (GtkListStore *store, const char * dir){
	DIR *dp = NULL;
	FILE *fp;
	struct dirent *de;
	char *icon = NULL;
	char *name = NULL;
	char *cmd = NULL;
	char *filename = NULL;
		
	dp = opendir (dir);
	
	if (dp) {
		while ((de = readdir (dp))) {
			
			filename = g_strdup_printf ("%s/%s", dir, de->d_name);
			
			if (strstr (filename, "desktop")) {
				fp = ppnl_open_file (filename);
				icon = ppnl_read_parameter ("Icon", fp);
				name = ppnl_read_parameter ("Name", fp);
				cmd = ppnl_read_parameter ("Exec", fp);
				g_strstrip (icon);
				g_strstrip (name);
				g_strstrip (cmd);
				
				add_item_to_store (store, icon, name, cmd);
				
				ppnl_close_file (fp);
			}

			free (icon);
			free (name);
			free (cmd);
			free (filename);
		}
		
		closedir (dp);
	}
}


void create_main_window (void){	
	GtkWidget *window = NULL;
	GtkWidget *button = NULL;
	GtkWidget *icon_view = NULL;
	GtkListStore *store = NULL;
	GtkWidget *banner = NULL;
	GdkPixbuf *pxbuf;
	window = gtk_dialog_new();
	gtk_window_set_title (GTK_WINDOW (window), "Simon Tatham's Portable Puzzle Collection");

	
	gtk_window_resize (GTK_WINDOW (window), width, height);
	gtk_window_set_icon_name (GTK_WINDOW (window), "puzzlespanel");
	
	
	/* The banner */
	pxbuf = gdk_pixbuf_new_from_file_at_scale(_BANNER, width, -1, TRUE, NULL);
	banner = gtk_image_new_from_pixbuf (pxbuf);
	
	g_signal_connect(G_OBJECT(window),
			 "delete_event", G_CALLBACK(destroy), NULL);

	g_signal_connect(G_OBJECT(window),
			 "destroy", G_CALLBACK(destroy), NULL);
	
	/* We create and define the icon view */
	icon_view = gtk_icon_view_new ();
	gtk_icon_view_set_item_width (GTK_ICON_VIEW (icon_view), 80);
	gtk_icon_view_set_row_spacing (GTK_ICON_VIEW (icon_view), 20);
	gtk_icon_view_set_column_spacing (GTK_ICON_VIEW (icon_view), 5);
	
	store = gtk_list_store_new (3, GDK_TYPE_PIXBUF, G_TYPE_STRING, G_TYPE_STRING);
	
	gtk_icon_view_set_pixbuf_column (GTK_ICON_VIEW (icon_view), ICON_COLUMN);
	gtk_icon_view_set_text_column (GTK_ICON_VIEW (icon_view), TEXT_COLUMN);
	gtk_icon_view_set_model (GTK_ICON_VIEW (icon_view), GTK_TREE_MODEL (store));
	
	g_signal_connect (G_OBJECT (icon_view), "item-activated", 
					G_CALLBACK(icon_activated), store);

	display_icons_in_panel (store, desktop_dir);

	button = gtk_button_new_from_stock("gtk-close");
	gtk_signal_connect (GTK_OBJECT (button), "clicked", G_CALLBACK (destroy), NULL);
	gtk_container_add (GTK_CONTAINER (GTK_DIALOG (window)->action_area), button);
	
	gtk_box_pack_start (GTK_BOX (GTK_DIALOG (window)->vbox), banner, FALSE, FALSE, 5);
	gtk_box_pack_start (GTK_BOX (GTK_DIALOG (window)->vbox), icon_view, TRUE, TRUE, 5);
	gtk_widget_show_all (window);
}


int main (int argc, char **argv){
	/* I know it's archaic, as I am :)  */
	int i;	
	while ((i = getopt(argc, argv, "d:W:H:h")) != EOF){
		switch (i){
			case 'd':
				desktop_dir = optarg;
				break;
			case 'W':
				width = atoi(optarg);
				break;
			case 'H':
				height = atoi(optarg);
				break;
			case 'h':
				usage(); exit(EXIT_FAILURE);
		}
	}
	
	gtk_set_locale ();
	
	signal (SIGCHLD, child_exited);
	
	gtk_init (&argc, &argv);
	create_main_window ();
	gtk_main();
	
	exit (EXIT_SUCCESS);
}


